@@ -2,7 +2,7 @@ import typing
 import collections.abc
 
 from .. import contracts
-from .generic import shutdown_instance
+from .generic import success_shutdown_instance, failure_shutdown_instance
 from .provider import DependencyProvider
 
 
@@ -160,7 +160,10 @@ class DependencyContainer(contracts.DependencyContainer):
             _desired = contracts.EventType.ON_SHUTDOWN_FAILURE if exc_type else contracts.EventType.ON_SHUTDOWN_SUCCESS
             if record.type == _desired:
                 singleton_instance = self._get_from_registry(contract)
-                await shutdown_instance(record.instance, singleton_instance, exc_type)
+                if exc_type:
+                    await failure_shutdown_instance(record.instance, singleton_instance, exc_type)
+                else:
+                    await success_shutdown_instance(record.instance, singleton_instance)
 
     def _get_from_registry(
         self,
